import torch
import numpy as np
import torch.utils.data as data

from os import listdir
from os.path import join
import scipy.io as scio
from scipy import interpolate
from PIL import Image
from torch import nn
from eval import SAM

def scipy_misc_imresize(arr, size, interp='bilinear', mode=None):
	im = Image.fromarray(arr, mode=mode)
	ts = type(size)
	if np.issubdtype(ts, np.signedinteger):
		percent = size / 100.0
		size = tuple((np.array(im.size)*percent).astype(int))
	elif np.issubdtype(type(size), np.floating):
		size = tuple((np.array(im.size)*size).astype(int))
	else:
		size = (size[1], size[0])
	func = {'nearest': 0, 'lanczos': 1, 'bilinear': 2, 'bicubic': 3, 'cubic': 3}
	imnew = im.resize(size, resample=func[interp]) # 调用PIL库中的resize函数
	return np.array(imnew)


def is_image_file(filename):
    return any(filename.endswith(extension) for extension in ['.mat'])
        	             	            	            	       
     
class TrainsetFromFolder(data.Dataset):
    def __init__(self, dataset_dir):
        super(TrainsetFromFolder, self).__init__()
        self.image_filenames = [join(dataset_dir , x) for x in listdir(dataset_dir ) if is_image_file(x)]

    def __getitem__(self, index):
        mat = scio.loadmat(self.image_filenames[index], verify_compressed_data_integrity=False)
        input = mat['lr'].astype(np.float32)
        label = mat['hr'].astype(np.float32)

        return torch.from_numpy(input), torch.from_numpy(label)
        
    def __len__(self):
        return len(self.image_filenames)
     
class ValsetFromFolder(data.Dataset):
    def __init__(self, dataset_dir):
        super(ValsetFromFolder, self).__init__()
        self.image_filenames = [join(dataset_dir , x) for x in listdir(dataset_dir ) if is_image_file(x)]

    def __getitem__(self, index):
        mat = scio.loadmat(self.image_filenames[index])
        input = mat['ms'].astype(np.float32).permute(2, 0, 1)
        label = mat['gt'].astype(np.float32).permute(2, 0, 1)
        bicu_4 = mat['ms_bicubic'].astype(np.float32).permute(2, 0, 1)

        return torch.from_numpy(input), torch.from_numpy(label),torch.from_numpy(bicu_4)


    def __len__(self):
        return len(self.image_filenames)   



