import os
import numpy as np
import torch
from os import listdir
import torch.nn as nn
from torch.autograd import Variable
from option import  opt
from data_utils import is_image_file
import scipy.io as scio  
from eval import PSNR, SSIM, SAM
# from scipy.misc import imresize
from data_utils import scipy_misc_imresize
import pdb
import time
from DSSTnet import DSSTnet


def main():


    input_path = '/home3/wx/dataset/Beishan/train_2/Val/'
    out_path = '/home3/wx/dataset/Beishan/train_2/Val_out/'
    
    if not os.path.exists(out_path):
        os.makedirs(out_path)      
    PSNRs = []
    SSIMs = []
    SAMs = []


    if not os.path.exists(out_path):
        os.makedirs(out_path)
                    
    if opt.cuda:
        print("=> use gpu id: '{}'".format(opt.gpus))
        os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpus
        if not torch.cuda.is_available():
            raise Exception("No GPU found or Wrong gpu id, please run without --cuda")

    model = DSSTnet(opt)

    if opt.cuda:
        model = nn.DataParallel(model).cuda()    
        
    checkpoint  = torch.load(opt.model_name)
    model.load_state_dict(checkpoint['model'],False)
    model.eval()       
    
    images_name = [x for x in listdir(input_path) if is_image_file(x)]           
    start = time.time()
    for index in range(len(images_name)):

        mat = scio.loadmat(input_path + images_name[index]) 
        LR = mat['LR'].astype(np.float32).transpose(2,0,1)
        HR = mat['HR'].astype(np.float32).transpose(2,0,1)

        bic = np.zeros(HR.shape, dtype=np.float32)
        #
        for i in range(bic.shape[0]):
        #     # bic[i,:,:] = imresize(LR[i,:,:], (bic.shape[1], bic.shape[2]), 'bicubic', mode='F')
            bic[i,:,:] = scipy_misc_imresize(LR[i,:,:],(bic.shape[1], bic.shape[2]),'bicubic')
        with torch.no_grad():
            bicu = torch.from_numpy(bic).float().contiguous().view(1, -1, bic.shape[1], bic.shape[2])
        with torch.no_grad():
            input = torch.from_numpy(LR).float().contiguous().view(1, -1, LR.shape[1], LR.shape[2])
        SR = np.array(HR).astype(np.float32)
        # input = LR

        first_state = None
        for i in range(input.shape[1]):
            neigbor_l = []
            neigbor_r = []
            if i == 0:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == 1:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 0, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == input.shape[1] - 2:

                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))



            elif i == input.shape[1] - 1:
                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 4, :, :].data.unsqueeze(1))

            else:
                neigbor_l.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 2, :, :].data.unsqueeze(1))

            x = input[:, i, :, :]
            single_bicu = bicu[:, i, :, :]
            with torch.no_grad():
                left = torch.cat(neigbor_l, 1)
            with torch.no_grad():
                right = torch.cat(neigbor_r, 1)
            with torch.no_grad():
                output, first_state = model(x, left, right, single_bicu, first_state)
            SR[i, :, :] = output.cpu().data[0].numpy()

        SR[SR<0] = 0
        SR[SR>1.] = 1.
        psnr = PSNR(SR, HR)
        ssim = SSIM(SR, HR)
        sam = SAM(SR, HR)
        
        PSNRs.append(psnr)
        SSIMs.append(ssim)
        SAMs.append(sam)
        
        SR = SR.transpose(1,2,0)   
        HR = HR.transpose(1,2,0)
        	                    
        scio.savemat(out_path + images_name[index], {'HR': HR, 'SR':SR})
        print("===The {}-th picture=====PSNR:{:.3f}=====SSIM:{:.4f}=====SAM:{:.3f}====Name:{}".format(index+1,  psnr, ssim, sam, images_name[index]))
    end = time.time()
    print(f"Inference time: {(end - start) / len(images_name):.5f} seconds")

    print("=====averPSNR:{:.3f}=====averSSIM:{:.4f}=====averSAM:{:.3f}".format(np.mean(PSNRs), np.mean(SSIMs), np.mean(SAMs)))

if __name__ == "__main__":
    main()
