# coding:utf-8
import os
from option import opt
# os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpus
import torch
import torch.backends.cudnn as cudnn
import torch.nn as nn
import torch.optim as optim
from torch.autograd import Variable
from torch.utils.data import DataLoader
import numpy as np
# from tensorboardX import writer

# from doublemodel import pre_Net
from data_utils import *
from eval import PSNR, SSIM, SAM
from torch.optim.lr_scheduler import MultiStepLR
from data_utils import is_image_file
import torch.nn.functional as F
from os import listdir
# import cv2
from math import *
import scipy.io as scio
from data_utils import scipy_misc_imresize
from DSSTnet import DSSTnet

from torchsummary import summary



def main():
    if opt.cuda:
        print("=> Use GPU ID: '{}'".format(opt.gpus))
        os.environ["CUDA_VISIBLE_DEVICES"] = opt.gpus
        if not torch.cuda.is_available():
            raise Exception("No GPU found or Wrong gpu id, please run without --cuda")

    torch.manual_seed(opt.seed)
    if opt.cuda:
        torch.cuda.manual_seed(opt.seed)
    cudnn.benchmark = True

    # Loading datasets

    train_set = TrainsetFromFolder('/home3/wx/dataset/Harvard/Train/8/')
    train_loader = DataLoader(dataset=train_set, num_workers=opt.threads, batch_size=opt.batchSize, shuffle=True)



    model = DSSTnet(opt)
    criterion = torch.nn.L1Loss()

    if opt.cuda:
        model = nn.DataParallel(model).cuda()
        criterion = criterion.cuda()
    else:
        model = model.cpu()
    print('# parameters:', sum(param.numel() for param in model.parameters()))

    # Setting Optimizer
    optimizer = optim.Adam(model.parameters(), lr=opt.lr, betas=(0.9, 0.999), eps=1e-08)

    # optionally resuming from a checkpoint
    if opt.resume:
        if os.path.isfile(opt.resume):
            print("=> loading checkpoint '{}'".format(opt.resume))
            checkpoint = torch.load(opt.resume)
            opt.start_epoch = checkpoint['epoch'] + 1
            model.load_state_dict(checkpoint['model'])
            optimizer.load_state_dict(checkpoint['optimizer'])
        else:
            print("=> no checkpoint found at '{}'".format(opt.resume))

            # Setting learning rate
    scheduler = MultiStepLR(optimizer, milestones=[30, 60, 90, 140, 175], gamma=0.1, last_epoch=-1)

    # Training 
    for epoch in range(opt.start_epoch, opt.nEpochs + 1):
        print("Epoch = {}, lr = {}".format(epoch, optimizer.param_groups[0]["lr"]))
        train(train_loader, optimizer, model, criterion, epoch)

        val(model, epoch)
        scheduler.step()
        save_checkpoint(epoch, model, optimizer)


def train(train_loader, optimizer, model, criterion, epoch):
    model = nn.DataParallel(model).cuda()
    model.train()

    for iteration, batch in enumerate(train_loader, 1):
        with torch.no_grad():
            input, label = batch[0], batch[1]

        first_state = None
        bicu = F.interpolate(input, scale_factor=opt.upscale_factor, mode='bicubic')
        if opt.cuda:
            input = input.cuda()
            label = label.cuda()
            bicu = bicu.cuda()
        for i in range(input.shape[1]):
            neigbor_l = []
            neigbor_r = []

            if i == 0:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == 1:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 0, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == input.shape[1] - 2:

                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))



            elif i == input.shape[1] - 1:
                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 4, :, :].data.unsqueeze(1))

            else:
                neigbor_l.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 2, :, :].data.unsqueeze(1))

            x = input[:, i, :, :]
            single_label = label[:, i, :, :]
            bicu_ = bicu[:, i, :, :]
            with torch.no_grad():
                left = torch.cat(neigbor_l, 1)
            with torch.no_grad():
                right = torch.cat(neigbor_r, 1)
            optimizer.zero_grad()

            SR, first_state = model(x, left, right, bicu_, first_state)
            SR = SR.squeeze(1)

            loss = criterion(SR, single_label)

            torch.autograd.set_detect_anomaly(True)
            loss.backward(retain_graph=True)
            optimizer.step()

        if iteration % 20 == 0:
            print("===> Epoch[{}]({}/{}): Loss: {:.10f}".format(epoch, iteration, len(train_loader), loss.item()))



def val(model, epoch):
    model.eval()
    PSNRs = []
    SSIMs = []
    SAMs = []
    val_path = '/home3/wx/dataset/Harvard/Val/8/'
    images_name = [x for x in listdir(val_path) if is_image_file(x)]

    for index in range(len(images_name)):
        mat = scio.loadmat(val_path + images_name[index])
        LR = mat['lr'].astype(np.float32).transpose(2, 0, 1)
        HR = mat['hr'].astype(np.float32).transpose(2, 0, 1)

        bic = np.zeros(HR.shape, dtype=np.float32)
        # #
        for i in range(bic.shape[0]):
            bic[i, :, :] = scipy_misc_imresize(LR[i, :, :], (bic.shape[1], bic.shape[2]), 'bicubic')
        with torch.no_grad():
            bicu = torch.from_numpy(bic).float().contiguous().view(1, -1, bic.shape[1], bic.shape[2])
        with torch.no_grad():
            input = torch.from_numpy(LR).float().contiguous().view(1, -1, LR.shape[1], LR.shape[2])
        SR = np.array(HR).astype(np.float32)

        first_state = None

        if opt.cuda:
            input = input.cuda()
            bicu = bicu.cuda()

        for i in range(input.shape[1]):
            neigbor_l = []
            neigbor_r = []
            if i == 0:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == 1:

                neigbor_l.append(input[:, 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, 0, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, 4, :, :].data.unsqueeze(1))

            elif i == input.shape[1] - 2:

                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))



            elif i == input.shape[1] - 1:
                neigbor_l.append(input[:, i - 3, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i - 4, :, :].data.unsqueeze(1))

            else:
                neigbor_l.append(input[:, i - 2, :, :].data.unsqueeze(1))
                neigbor_l.append(input[:, i - 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 1, :, :].data.unsqueeze(1))
                neigbor_r.append(input[:, i + 2, :, :].data.unsqueeze(1))

            x = input[:, i, :, :]
            bicu_ = bicu[:, i, :, :]

            with torch.no_grad():
                left = torch.cat(neigbor_l, 1)
            with torch.no_grad():
                right = torch.cat(neigbor_r, 1)


            with torch.no_grad():
                SR, first_state = model(x, left, right, bicu_, first_state)

            SR[i, :, :] = SR.cpu().data[0].numpy()
        SR[SR < 0] = 0
        SR[SR > 1.] = 1.
        psnr1 = PSNR(SR, HR)
        ssim1 = SSIM(SR, HR)
        sam1 = SAM(SR, HR)


        PSNRs.append(psnr1)
        SSIMs.append(ssim1)
        SAMs.append(sam1)

        # print("PSNR:{:.3f}=====SSIM:{:.4f}=====SAM:{:.3f}".format(psnr1, ssim1, sam1))

    print("PSNR = {:.3f}   SSIM = {:.4F}    SAM = {:.3f}".format(np.mean(PSNRs), np.mean(SSIMs), np.mean(SAMs)))




def save_checkpoint(epoch, model, optimizer):
    model_out_path = "./checkpoint/" + "{}/{}_model_{}_epoch_{}.pth".format(opt.saveName,opt.datasetName,
                                                                                             opt.upscale_factor, epoch)
    state = {"epoch": epoch, "model": model.state_dict(), "optimizer": optimizer.state_dict()}
    if not os.path.exists("./checkpoint/"+"{}/".format(opt.saveName)):
        os.makedirs("./checkpoint/"+"{}/".format(opt.saveName))
    torch.save(state, model_out_path)


if __name__ == "__main__":
    main()
